/*
Sakul - http://wwww.sakul.cz - SakulRaider@seznam.cz (2016)
Program pro měření CO2 pomocí čidla Figaro CDM4161A
Určeno pro verzi hardwaru 1.0
Verze programu 1.1:
v1.1 - Přidáno zobrazování na LCD 2x16 znaků
     - Plus to posílá nějaká data na UART
v1.0 - Očesaná verze do kanclu
:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
Určeno pro ATMEGA8-16PU                    (ATMEGA328P-PU (Arduino UNO))
Vývojové prostředí Arduino IDE 1.6.5
LED Bargraf (signalizace):
Zelená  LED1 - 400ppm
        LED2 - 600ppm
        LED3 - 800ppm
        LED4 - 1000ppm
        LED5 - 1200ppm
        LED6 - 1400ppm
        LED7 - 1600ppm
        LED8 - 1800ppm
        LED9 - 2000ppm
        LED10 - 2200ppm
Při zahřívání čidla blikat LED5 a LED6

:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
DODĚLAT:
- Snad nic zásadního

 */
// Knihovny ----------------------------------------------------------------------------------
#include <Wire.h>
// #include <LCD.h>                        
#include <LiquidCrystal_I2C.h>

// Konfigurace knihoven ----------------------------------------------------------------------
// Addr, En, Rw, Rs, d4, d5, d6, d7, backlighpin, polarity
// LiquidCrystal_I2C lcd(0x27, 2, 1, 0, 4, 5, 6, 7, 3, POSITIVE);      // Pro I2C desky s adresou 27 (Modrý trimr kontrastu)
  LiquidCrystal_I2C lcd(0x20, 4, 5, 6, 0, 1, 2, 3, 7, NEGATIVE);      // Pro I2C desky s adresou 20 (SMD trimr kontrastu)

// Definice proměných ------------------------------------------------------------------------
const int LED1 =  4;                       // Výstupy na LEDky 400ppm
const int LED2 =  13;                      // Výstupy na LEDky 600ppm
const int LED3 =  12;                      // Výstupy na LEDky 800ppm
const int LED4 =  5;                       // Výstupy na LEDky 1000ppm
const int LED5 =  11;                      // Výstupy na LEDky 1200ppm
const int LED6 =  6;                       // Výstupy na LEDky 1400ppm
const int LED7 =  10;                      // Výstupy na LEDky 1600ppm
const int LED8 =  7;                       // Výstupy na LEDky 1800ppm
const int LED9 =  9;                       // Výstupy na LEDky 2000ppm
const int LED10 = 8;                       // Výstupy na LEDky 2200ppm

const int LedAlarm = 2;                    // Výstup na LED signalizující ALARM koncentrace CO2 (Při překročení 1000ppm se rozsviti a při poklesu pod 900ppm zhasne
const int LED = 3;                         // Zatím pro signalizaci, že je čidlo v pořádku

const int ADCin1 =  A1;                    // Vstup převodníku Analog/digital (Hodnota U pro CO2 - Vconc)
const int ADCin2 =  A0;                    // Vstup převodníku Analog/digital (Hodnota U pro výstup - CTRL)
float ADC1;                                // Proměnná na "Čistou" hodnotu z AD převodníku 0-1023 (Pro Vconc)
float ADC2;                                // Proměnná na "Čistou" hodnotu z AD převodníku 0-1023 (Pro CTRL)
int KonstVconc = 205;                      // Konstanta pro převod ADC na volty (0-5V = 0-1023ADC)  (Default 5V=205)
int KonstCTRL = 205;                       // Konstanta pro převod ADC na volty (0-5V = 0-1023ADC)  (Default 5V=205)
float Vconc;                               // Proměnná se skutečným napětím na vstupu Vconc
float CTRL;                                // Proměnná se skutečným napětím na vstupu CTRL
float PPM;                                 // Hodnota CO2 (ppm)

long lastJob1;                             // Pomocná proměnná pro časování
long lastJob2;                             // Pomocná proměnná pro časování
long lastJob3;                             // Pomocná proměnná pro časování
long lastJob4;                             // Pomocná proměnná pro časování

boolean Timing = false;                    // Pomocná proměnná blikače

// Nastavení procesoru -----------------------------------------------------------------------
void setup() {
  Serial.begin(9600);                      // Nastavení komunikační rychlosti UARTu
  pinMode(LED1, OUTPUT);                   // Definice vstupních portů
  pinMode(LED2, OUTPUT);
  pinMode(LED3, OUTPUT);
  pinMode(LED4, OUTPUT);
  pinMode(LED5, OUTPUT);
  pinMode(LED6, OUTPUT);
  pinMode(LED7, OUTPUT);
  pinMode(LED8, OUTPUT);
  pinMode(LED9, OUTPUT);
  pinMode(LED10, OUTPUT);
  pinMode(LedAlarm, OUTPUT);
  pinMode(LED, OUTPUT);

  lcd.begin(16,2);                         // Nastavení displeje 16 znaků na 2 řádky
  lcd.backlight();                         // Zapnout podsvícení displeje
  lcd.setCursor(0, 0);                     // Nastavit kurzor na první řádek a první znak
  lcd.print("www.sakul.cz");               // Nápis na první řádek
  lcd.setCursor(0, 1);                     // Nastavit kurzor na druhý řádek a druhý znak
  lcd.print("Firmware 1.0");

// lcd.noBlink();                         
 delay (9000);
 lcd.clear();
}

// Hlavní program ----------------------------------------------------------------------------
// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
void loop() {
 MereniVconc();                           
 MereniCTRL();                            
 Bargraf();                              
 NaLCD();                                 
 Ledky();                                 

 UART();                                 
}

// :::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
// Podprogramy -------------------------------------------------------------------------------
void MereniVconc(){
  ADC1 = analogRead (ADCin1);               
  Vconc = ADC1 / KonstVconc;               
  PPM = Vconc * 1000;                      
}
// ------------------------------------------------------------------------------------------- 
void MereniCTRL(){
  ADC2 = analogRead (ADCin2);               
  CTRL = ADC2 / KonstCTRL;                  
}
// ------------------------------------------------------------------------------------------- 
void Bargraf(){
  
if (millis() > (600 + lastJob2)){          
    if (Timing == false){
        Timing = true;    
       }
    else Timing = false;
   
    lastJob2 = millis();
}
  
if (millis() > (300 + lastJob3)){         
  
if (PPM < 4400){                          
 if (PPM >= 400){                         
    digitalWrite(LED1,HIGH);
    }
    else {digitalWrite(LED1,LOW);}

 if (PPM >= 600){
    digitalWrite(LED2,HIGH);
    }
    else {digitalWrite(LED2,LOW);}

 if (PPM >= 800){
    digitalWrite(LED3,HIGH);
    }
    else {digitalWrite(LED3,LOW);}

 if (PPM >= 1000){
    digitalWrite(LED4,HIGH);
    }
    else {digitalWrite(LED4,LOW);}

 if (PPM >= 1200){
    digitalWrite(LED5,HIGH);
    }
    else {digitalWrite(LED5,LOW);}

 if (PPM >= 1400){
    digitalWrite(LED6,HIGH);
    }
    else {digitalWrite(LED6,LOW);}

 if (PPM >= 1600){
    digitalWrite(LED7,HIGH);
    }
    else {digitalWrite(LED7,LOW);}

 if (PPM >= 1800){
    digitalWrite(LED8,HIGH);
    }
    else {digitalWrite(LED8,LOW);}

  if (PPM >= 2000){
    digitalWrite(LED9,HIGH);
    }
    else {digitalWrite(LED9,LOW);}

   if (PPM >= 2200){
    digitalWrite(LED10,HIGH);
    }
    else {digitalWrite(LED10,LOW);}
    
}
   lastJob3 = millis();
 }
 
 if (PPM > 4400){                          
  if (Timing == true){
//    digitalWrite(LED1,HIGH);
//    digitalWrite(LED2,HIGH);
//    digitalWrite(LED3,HIGH);
//    digitalWrite(LED4,HIGH);
    digitalWrite(LED5,HIGH);
    digitalWrite(LED6,HIGH);
//    digitalWrite(LED7,HIGH);
//    digitalWrite(LED8,HIGH);
//    digitalWrite(LED9,HIGH);
//    digitalWrite(LED10,HIGH);
    }
    else {
         digitalWrite(LED1,LOW);
         digitalWrite(LED2,LOW);
         digitalWrite(LED3,LOW);
         digitalWrite(LED4,LOW);
         digitalWrite(LED5,LOW);
         digitalWrite(LED6,LOW);
         digitalWrite(LED7,LOW);
         digitalWrite(LED8,LOW);
         digitalWrite(LED9,LOW);
         digitalWrite(LED10,LOW);       
         }
   }
}
// -------------------------------------------------------------------------------------------
void UART(){
 if (millis() > (3000 + lastJob1)){        

 Serial.print ("ADC1: ");                  
 Serial.print (ADC1);
 Serial.print (" //  Vconc: ");
 Serial.print (Vconc);
 Serial.print ("V");
 Serial.print (" //  CO2: ");
 Serial.print (PPM,0);                     
 Serial.print ("ppm");
 Serial.print (" //  CTRL: ");
 Serial.print (CTRL,3);
 Serial.println ("V");
 /*
 
 */
  
 lastJob1 = millis();
 } 
}
// ------------------------------------------------------------------------------------------- 
void NaLCD(){
  if (millis() > (1000 + lastJob4)){        

  if (PPM < 1000){                          
  lcd.setCursor(3, 0);                      
  lcd.print("Mereni CO2");                  
  lcd.setCursor(15, 1);
  lcd.print(" ");           
  lcd.setCursor(0, 1);                     
  lcd.print("Hodnota ppm=");
  lcd.print(PPM,0);
   }
   else if (PPM < 4400){                    
  lcd.setCursor(3, 0);                     
  lcd.print("Mereni CO2");           
  lcd.setCursor(0, 1);                     
  lcd.print("Hodnota ppm=");
  lcd.print(PPM,0);
    }
   else{  
  lcd.setCursor(3, 0);                        
  lcd.print("Mereni CO2");
  lcd.setCursor(0, 1);                     
  lcd.print("Zahrivam senzor ");
   }

  lastJob4 = millis();
 }
}
// ------------------------------------------------------------------------------------------- 
void Ledky(){
  if (CTRL > 0.6){                          
    digitalWrite(LedAlarm,HIGH);
  }
  else {digitalWrite(LedAlarm,LOW);}        

//  if (Vconc < 0.2){                         
//    digitalWrite(LED,HIGH);                 
//  }
//  else {digitalWrite(LED,LOW);}             
//------------------------------------
  if (PPM >= 1500){                        
    digitalWrite(LED,HIGH);
    }
    else {digitalWrite(LED,LOW);}

}
// ------------------------------------------------------------------------------------------- 
